import { CommonModule } from '@angular/common';
import { __awaiter } from 'tslib';
import { Router } from '@angular/router';
import { JwtHelperService } from '@auth0/angular-jwt';
import { NgForage } from 'ngforage';
import { HttpClient, HttpParams, HttpRequest } from '@angular/common/http';
import { from } from 'rxjs';
import { switchMap } from 'rxjs/operators';
import { InjectionToken, NgModule, Optional, SkipSelf, Injectable, Inject, defineInjectable, inject } from '@angular/core';
import { some, cloneDeep, omit, forOwn, isArray, mapValues, isBoolean } from 'lodash';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const EnvironmentConfigService = new InjectionToken('EnvironmentConfig');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ApiLibModule {
    /**
     * @param {?} parentModule
     */
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('ApiModule is already loaded. Import it in the AppModule only');
        }
    }
    /**
     * @param {?} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: ApiLibModule,
            providers: [
                { provide: EnvironmentConfigService, useValue: config },
            ]
        };
    }
}
ApiLibModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [
                    CommonModule
                ],
                exports: []
            },] }
];
/** @nocollapse */
ApiLibModule.ctorParameters = () => [
    { type: ApiLibModule, decorators: [{ type: Optional }, { type: SkipSelf }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
const HttpMethods = {
    GET: 'GET',
    POST: 'POST',
    PUT: 'PUT',
    PATCH: 'PUT',
    DELETE: 'DELETE',
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
const RequestTypes = {
    GET: 0,
    SEARCH: 1,
    ADD: 2,
    UPDATE: 3,
    DELETE: 4,
    RESTORE: 5,
};
RequestTypes[RequestTypes.GET] = 'GET';
RequestTypes[RequestTypes.SEARCH] = 'SEARCH';
RequestTypes[RequestTypes.ADD] = 'ADD';
RequestTypes[RequestTypes.UPDATE] = 'UPDATE';
RequestTypes[RequestTypes.DELETE] = 'DELETE';
RequestTypes[RequestTypes.RESTORE] = 'RESTORE';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const methodsWithBody = [HttpMethods.POST, HttpMethods.PUT, HttpMethods.PATCH];
/** @type {?} */
const uploadingSection = ['media'];
class HttpService {
    /**
     * @param {?} config
     * @param {?} http
     */
    constructor(config, http) {
        this.config = config;
        this.http = http;
    }
    /**
     * @private
     * @param {?} urlParams
     * @param {?} method
     * @param {?} options
     * @return {?}
     */
    handle(urlParams, method, options) {
        options = this.filterParams(urlParams, options);
        if (methodsWithBody.indexOf(method) === -1) {
            options = {
                params: new HttpParams({
                    fromObject: options
                })
            };
        }
        /** @type {?} */
        let request = new HttpRequest(method, this.buildUrl(urlParams), options, {
            responseType: urlParams.responseType || 'json'
        });
        // add header to prevent token checking for public requests
        if (urlParams.insecure === true) {
            request = request.clone({ setHeaders: { Insecure: '1' } });
            delete urlParams.insecure;
        }
        return this.http.request(request)
            .toPromise()
            .then((/**
         * @param {?} event
         * @return {?}
         */
        (event) => {
            // console.log('response', event);
            return {
                result: true,
                data: (urlParams.rawResponse === true) ? event : event.body
            };
        }))
            .catch((/**
         * @param {?} error
         * @return {?}
         */
        (error) => {
            return {
                result: false,
                error: (urlParams.rawResponse === true) ? error : error.error
            };
        }));
    }
    /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    get(urlParams, params = {}) {
        return this.handle(urlParams, HttpMethods.GET, params);
    }
    /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    search(urlParams, params = {}) {
        return this.handle(urlParams, HttpMethods.GET, params);
    }
    /**
     * @param {?} urlParams
     * @param {?} data
     * @return {?}
     */
    save(urlParams, data) {
        return this.handle(urlParams, HttpMethods.POST, data);
    }
    /**
     * @param {?} urlParams
     * @param {?} data
     * @return {?}
     */
    update(urlParams, data) {
        return this.handle(urlParams, HttpMethods.PUT, data);
    }
    /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    delete(urlParams, params = {}) {
        return this.handle(urlParams, HttpMethods.DELETE, params);
    }
    /**
     * Transform keys to `key[]` for array type vales
     * and remove params with empty values
     * @private
     * @param {?} urlParams
     * @param {?} options
     * @return {?}
     */
    filterParams(urlParams, options) {
        /** @type {?} */
        let params = {};
        forOwn(options, (/**
         * @param {?} value
         * @param {?} key
         * @return {?}
         */
        (value, key) => {
            if (isArray(value) && key.indexOf('[]') === -1) {
                key += '[]';
            }
            if (value !== null && value !== undefined) {
                params[key] = value;
            }
        }));
        // convert boolean values to numeric except media content
        if (uploadingSection.indexOf(urlParams.section) === -1) {
            params = mapValues(params, (/**
             * @param {?} value
             * @return {?}
             */
            value => isBoolean(value) ? +value : value));
        }
        return params;
    }
    /**
     * Make url like `/section/id/cmd/secondId`
     * @private
     * @param {?} urlParams
     * @return {?}
     */
    buildUrl(urlParams) {
        /** @type {?} */
        const url = [`${this.config.url}/${urlParams.section}`];
        if (urlParams.id !== undefined) {
            url.push(`${urlParams.id}`);
        }
        if (urlParams.cmd) {
            url.push(`${urlParams.cmd}`);
        }
        if (urlParams.secondId) {
            url.push(`${urlParams.secondId}`);
        }
        return url.join('/');
    }
}
HttpService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
HttpService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [EnvironmentConfigService,] }] },
    { type: HttpClient }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AuthService {
    /**
     * @param {?} httpService
     * @param {?} router
     * @param {?} storageService
     */
    constructor(httpService, router, storageService) {
        this.httpService = httpService;
        this.router = router;
        this.storageService = storageService;
        this.jwtHelper = new JwtHelperService();
        this._user = null;
    }
    /**
     * @return {?}
     */
    getToken() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.storageService.getItem('jwt');
        });
    }
    /**
     * @param {?} token
     * @return {?}
     */
    setToken(token) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.storageService.setItem('jwt', token);
        });
    }
    /**
     * Set current logged user
     * @param {?} user
     * @return {?}
     */
    setUser(user) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.storageService.setItem('user', user);
        });
    }
    /**
     * @return {?}
     */
    getUser() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._user) {
                /** @type {?} */
                const user = yield this.storageService.getItem('user');
                this._user = user;
                return user;
            }
            else {
                return this._user;
            }
        });
    }
    /**
     * Check if current user is authenticated
     * @return {?}
     */
    isAuthenticated() {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const token = yield this.getToken();
            return token !== null && !this.isExpired();
        });
    }
    /**
     * Check if token is expired
     * @return {?}
     */
    isExpired() {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const token = yield this.getToken();
            return token !== null && this.jwtHelper.isTokenExpired(token);
        });
    }
    /**
     * Attempt to refresh token
     * @return {?}
     */
    refreshToken() {
        return this.httpService
            .search({
            section: 'auth',
            cmd: 'refresh',
            rawResponse: true,
            insecure: true
        })
            .then((/**
         * @param {?} response
         * @return {?}
         */
        (response) => __awaiter(this, void 0, void 0, function* () {
            if (response.result) {
                /** @type {?} */
                const newToken = response.data.headers.get('authorization').split(' ')[1];
                /** @type {?} */
                const claims = this.jwtHelper.decodeToken(newToken);
                yield this.setUser(claims.user);
                yield this.setToken(newToken);
                return newToken;
            }
            else {
                return '';
            }
        })));
    }
    /**
     * Clear all session variables
     * @return {?}
     */
    logout() {
        return __awaiter(this, void 0, void 0, function* () {
            this._user = null;
            yield this.storageService.removeItem('jwt');
            yield this.storageService.removeItem('user');
        });
    }
}
AuthService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
AuthService.ctorParameters = () => [
    { type: HttpService },
    { type: Router },
    { type: NgForage }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ApiAuthInterceptor {
    /**
     * @param {?} config
     * @param {?} authService
     */
    constructor(config, authService) {
        this.config = config;
        this.authService = authService;
    }
    /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    intercept(request, next) {
        return from(this.authService.getToken())
            .pipe(switchMap((/**
         * @param {?} token
         * @return {?}
         */
        token => {
            if (token) {
                request = request.clone({ setHeaders: { Authorization: `Bearer ${token}` } });
            }
            if (!this.isWhitelistedDomain(request)) {
                return next.handle(null);
            }
            return next.handle(request);
        })));
    }
    /**
     * @private
     * @param {?} request
     * @return {?}
     */
    isWhitelistedDomain(request) {
        if (this.config.domains !== undefined) {
            return some(this.config.domains, (/**
             * @param {?} domain
             * @return {?}
             */
            domain => {
                /** @type {?} */
                const hostname = new URL(request.url).hostname.toLowerCase();
                return hostname.match(new RegExp(domain.toLowerCase(), 'i')) !== null;
            }));
        }
        return true;
    }
}
ApiAuthInterceptor.decorators = [
    { type: Injectable }
];
/** @nocollapse */
ApiAuthInterceptor.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [EnvironmentConfigService,] }] },
    { type: AuthService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ApiJwtInterceptor {
    /**
     * @param {?} authService
     */
    constructor(authService) {
        this.authService = authService;
        this.tokenUpdate = false;
    }
    /**
     * @param {?} request
     * @param {?} next
     * @return {?}
     */
    intercept(request, next) {
        return from(this.authService.isExpired())
            .pipe(switchMap((/**
         * @param {?} isExpired
         * @return {?}
         */
        isExpired => {
            if (!this.insecureUrl(request) && isExpired) {
                this.tokenUpdate = true;
                return from(this.authService
                    .refreshToken()
                    .then((/**
                 * @param {?} newToken
                 * @return {?}
                 */
                newToken => {
                    if (newToken) {
                        this.tokenUpdate = false;
                        return next.handle(request).toPromise();
                    }
                    else {
                        this.tokenUpdate = false;
                        this.authService.logout();
                        return next.handle(null).toPromise();
                    }
                }))
                    .catch((/**
                 * @param {?} result
                 * @return {?}
                 */
                (result) => {
                    if (this.tokenUpdate) {
                        this.tokenUpdate = false;
                        this.authService.logout();
                    }
                    return next.handle(null).toPromise();
                })));
            }
            else {
                return next.handle(request);
            }
        })));
    }
    /**
     * @private
     * @param {?} request
     * @return {?}
     */
    insecureUrl(request) {
        return request.headers.has('Insecure');
    }
}
ApiJwtInterceptor.decorators = [
    { type: Injectable }
];
/** @nocollapse */
ApiJwtInterceptor.ctorParameters = () => [
    { type: AuthService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 * @template T
 */
class PostRequest {
    /**
     * @param {?} service
     * @param {?} entity
     */
    constructor(service, entity) {
        this.service = service;
        this.entity = entity;
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} urlParams
     * @return {THIS}
     */
    url(urlParams) {
        (/** @type {?} */ (this)).urlParams = urlParams;
        return (/** @type {?} */ (this));
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} data
     * @return {THIS}
     */
    data(data) {
        (/** @type {?} */ (this)).entity = data;
        return (/** @type {?} */ (this));
    }
    // filter(filter: ((data: any) => T) | string[]): ApiBodyRequest<T> {
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} filter
     * @return {THIS}
     */
    filter(filter) {
        if (typeof filter === 'function') {
            (/** @type {?} */ (this)).entity = filter((/** @type {?} */ (this)).entity);
        }
        else {
            (/** @type {?} */ (this)).entity = (/** @type {?} */ (omit((/** @type {?} */ (this)).entity, filter)));
        }
        return (/** @type {?} */ (this));
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class ApiAddRequest extends PostRequest {
    /**
     * @return {?}
     */
    execute() {
        return this.service.add(this.entity, this.urlParams);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 * @template T
 */
class GetRequest {
    /**
     * @param {?} service
     * @param {?} queryParams
     */
    constructor(service, queryParams) {
        this.service = service;
        this.queryParams = queryParams;
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} urlParams
     * @return {THIS}
     */
    url(urlParams) {
        (/** @type {?} */ (this)).urlParams = urlParams;
        return (/** @type {?} */ (this));
    }
    /**
     * Implementation of the adding `with_trashed` into request
     *
     * @template THIS
     * @this {THIS}
     * @param {?=} value
     * @return {THIS}
     */
    withTrashed(value = true) {
        (/** @type {?} */ (this)).queryParams.with_trashed = value;
        return (/** @type {?} */ (this));
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class ApiGetRequest extends GetRequest {
    /**
     * @param {?} service
     * @param {?} queryParams
     */
    constructor(service, queryParams) {
        super(service, queryParams);
        this.service = service;
        this.queryParams = queryParams;
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} id
     * @return {THIS}
     */
    id(id) {
        (/** @type {?} */ (this)).queryParams.id = id;
        return (/** @type {?} */ (this));
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    with(value) {
        (/** @type {?} */ (this)).queryParams.with = value;
        return (/** @type {?} */ (this));
    }
    /**
     * @return {?}
     */
    execute() {
        /** @type {?} */
        const params = cloneDeep(this.queryParams);
        delete params.id;
        return this.service.get(this.queryParams.id, params || {});
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class ApiSearchRequest extends GetRequest {
    /**
     * @param {?} service
     * @param {?} queryParams
     */
    constructor(service, queryParams) {
        super(service, queryParams);
        this.service = service;
        this.queryParams = queryParams;
    }
    // TODO: try to use decorator to change url (as parameter of function) + use declare in service
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    query(value) {
        (/** @type {?} */ (this)).queryParams.query = value;
        return (/** @type {?} */ (this));
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    page(value) {
        (/** @type {?} */ (this)).queryParams.page = value;
        return (/** @type {?} */ (this));
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    perPage(value) {
        (/** @type {?} */ (this)).queryParams.per_page = value;
        return (/** @type {?} */ (this));
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} field
     * @param {?} value
     * @return {THIS}
     */
    filterBy(field, value) {
        (/** @type {?} */ (this)).queryParams[field] = value;
        return (/** @type {?} */ (this));
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} field
     * @param {?=} isDescending
     * @return {THIS}
     */
    orderBy(field, isDescending = false) {
        (/** @type {?} */ (this)).queryParams.order_by = field;
        (/** @type {?} */ (this)).queryParams.desc = isDescending;
        return (/** @type {?} */ (this));
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?=} value
     * @return {THIS}
     */
    all(value = true) {
        (/** @type {?} */ (this)).queryParams.all = value;
        return (/** @type {?} */ (this));
    }
    /**
     * @template THIS
     * @this {THIS}
     * @param {?} value
     * @return {THIS}
     */
    with(value) {
        (/** @type {?} */ (this)).queryParams.with = value;
        return (/** @type {?} */ (this));
    }
    /**
     * @return {?}
     */
    execute() {
        return this.service.search(this.queryParams, this.urlParams);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class ApiUpdateRequest extends PostRequest {
    /**
     * @return {?}
     */
    execute() {
        return this.service.update(this.entity, this.urlParams);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @abstract
 * @template T
 */
class ApiService {
    /**
     * @param {?} httpService
     */
    constructor(httpService) {
        this.httpService = httpService;
        this.section = '';
        this.omittedFields = [];
        this.isInsecure = false;
    }
    /**
       *
       * @param {number | string} id
       * @param {object} params
       * @param {UrlParams} urlParams
       * @return {Promise<ApiResponse<Y>>}
       */
    // get<Y>(id: number | string, params?: Omit<GetParams, 'id'>, urlParams?: UrlParams): Promise<ApiResponse<Y>>;
    /**
     *
     * @param {?} id
     * @param {?=} params
     * @param {?=} urlParams
     * @return {?}
     */
    get(id, params, urlParams) {
        /** @type {?} */
        const url = urlParams || {
            section: this.section,
            id: id,
            insecure: this.isInsecure
        };
        return this.httpService.get(url, params);
    }
    /**
       * @param {Y} data
       * @param {UrlParams} urlParams
       * @return {Promise<ApiResponse<Y>>}
       */
    // add<Y extends ApiEntity>(data: Partial<Y>, urlParams?: UrlParams): Promise<ApiResponse<T>>;
    /**
     *
     * @param {?} data
     * @param {?=} urlParams
     * @return {?}
     */
    add(data, urlParams) {
        /** @type {?} */
        const url = urlParams || {
            section: this.section,
            insecure: this.isInsecure
        };
        return this.httpService.save(url, this.omitFields(data));
    }
    /**
       *
       * @param {Y} data
       * @param {UrlParams} urlParams
       * @return {Promise<ApiResponse<void>>}
       */
    // update<Y extends ApiEntity>(data: Partial<Y>, urlParams?: UrlParams): Promise<ApiResponse<void>>;
    /**
     *
     * @param {?} data
     * @param {?=} urlParams
     * @return {?}
     */
    update(data, urlParams) {
        /** @type {?} */
        const url = urlParams || {
            section: this.section,
            id: data.id,
            insecure: this.isInsecure
        };
        /** @type {?} */
        const dataWithoutId = cloneDeep(data);
        delete dataWithoutId.id;
        return this.httpService.update(url, dataWithoutId);
    }
    /**
     *
     * @param {?} id
     * @param {?=} urlParams
     * @return {?}
     */
    delete(id, urlParams) {
        /** @type {?} */
        const url = urlParams || {
            section: this.section,
            id: id,
            insecure: this.isInsecure
        };
        return this.httpService.delete(url);
    }
    /**
     * Soft delete list of objects
     * (if backend supports this method)
     * @param {?} ids
     * @param {?=} urlParams
     * @return {?}
     */
    deleteGroup(ids, urlParams) {
        /** @type {?} */
        const url = urlParams || {
            section: this.section,
            insecure: this.isInsecure
        };
        return this.httpService
            .delete(url, {
            'ids[]': ids
        });
    }
    /**
     *
     * @param {?} id
     * @param {?=} urlParams
     * @return {?}
     */
    restore(id, urlParams) {
        /** @type {?} */
        const url = urlParams || {
            section: this.section,
            id: id,
            cmd: 'restore',
            insecure: this.isInsecure
        };
        return this.httpService.save(url, {});
    }
    /**
     * Restore list of objects
     * (if backend supports this method)
     * @param {?} ids
     * @param {?=} urlParams
     * @return {?}
     */
    restoreGroup(ids, urlParams) {
        /** @type {?} */
        const url = urlParams || {
            section: this.section,
            id: 'restore',
            insecure: this.isInsecure
        };
        return this.httpService
            .save(url, {
            'ids[]': ids
        });
    }
    /**
       *
       * @param {SearchParams} searchParams
       * @param {UrlParams} urlParams
       * @return {Promise<ApiResponse<Y[]>>}
       */
    // search<Y extends ApiEntity>(searchParams: SearchParams, urlParams?: UrlParams): Promise<ApiResponse<ApiPagination<Y[]>>>;
    /**
     *
     * @param {?} searchParams
     * @param {?=} urlParams
     * @return {?}
     */
    search(searchParams, urlParams) {
        /** @type {?} */
        const url = urlParams || {
            section: this.section,
            insecure: this.isInsecure
        };
        return this.httpService
            .search(url, searchParams)
            .then((/**
         * @param {?} response
         * @return {?}
         */
        response => {
            if (searchParams && searchParams.all) {
                // FIXME: temporary solution until backend will be updated to the new version
                response.data = {
                    current_page: 1,
                    data: response.data,
                    first_page_url: '',
                    from: 1,
                    last_page: 1,
                    last_page_url: '',
                    next_page_url: '',
                    path: '',
                    per_page: response.data.length,
                    prev_page_url: '',
                    to: response.data.length,
                    total: response.data.length
                };
            }
            return response;
        }));
    }
    /**
     * @param {?} method
     * @param {?=} params
     * @return {?}
     */
    request(method, params = {}) {
        /** @type {?} */
        let request;
        switch (method) {
            case RequestTypes.SEARCH: {
                request = new ApiSearchRequest(this, params);
                break;
            }
            case RequestTypes.GET: {
                request = new ApiGetRequest(this, params);
                break;
            }
            case RequestTypes.ADD: {
                request = new ApiAddRequest(this, params);
                break;
            }
            case RequestTypes.UPDATE: {
                request = new ApiUpdateRequest(this, params);
                break;
            }
        }
        return request;
    }
    /**
     * @protected
     * @template Y
     * @param {?} data
     * @return {?}
     */
    omitFields(data) {
        return omit(data, ...this.omittedFields);
    }
}
ApiService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
/** @nocollapse */
ApiService.ctorParameters = () => [
    { type: HttpService }
];
/** @nocollapse */ ApiService.ngInjectableDef = defineInjectable({ factory: function ApiService_Factory() { return new ApiService(inject(HttpService)); }, token: ApiService, providedIn: "root" });

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { ApiLibModule, HttpMethods, RequestTypes, ApiAuthInterceptor, ApiJwtInterceptor, ApiAddRequest, ApiGetRequest, ApiSearchRequest, ApiUpdateRequest, GetRequest, PostRequest, ApiService, AuthService, EnvironmentConfigService, HttpService };

//# sourceMappingURL=api-lib.js.map